@@ -12,6 +12,7 @@
 from libpwman.cryptsql import *
 from libpwman.exception import *
 from libpwman.util import *
+import libpwman.otp
 
 import csv
 import io
@@ -80,6 +81,11 @@ class PWManEntryTOTP:
 	entry		: PWManEntry = None
 	totpId		: int = None
 
+	def generate(self):
+		return libpwman.otp.totp(key=self.key,
+					 nrDigits=self.digits,
+					 hmacHash=self.hmacHash)
+
 class PWManDatabase(CryptSQL):
 	"""Encrypted pwman database.
 	"""
@@ -225,7 +231,7 @@ class PWManDatabase(CryptSQL):
 				 "LIMIT 1);",
 				 (category,))
 		data = c.fetchOne()
-		return data and data[0]
+		return bool(data) and bool(data[0])
 
 	def getCategoryNames(self):
 		"""Get all category names in the database.
@@ -375,7 +381,8 @@ class PWManDatabase(CryptSQL):
 		      "FROM entries "\
 		      "WHERE entries.id IN ( "
 		sql += ", ".join("?" for ID in IDs)
-		sql += " );"
+		sql += " ) "
+		sql += "ORDER BY entries.category, entries.title;"
 		params = [ str(ID) for ID in IDs ]
 		dump(sql, params)
 		c = self.sqlExec(sql, params)
@@ -400,7 +407,7 @@ class PWManDatabase(CryptSQL):
 				 (category,
 				  title))
 		data = c.fetchOne()
-		return data and data[0]
+		return bool(data) and bool(data[0])
 
 	def addEntry(self, entry):
 		"""Create a new entry in the database.
@@ -688,7 +695,8 @@ class PWManDatabase(CryptSQL):
 		c = self.sqlExec("SELECT entryattr.id, entryattr.name, entryattr.data "
 				 "FROM entryattr, entries "
 				 "WHERE entries.category=? AND entries.title=? AND "
-				 "entryattr.entry = entries.id;",
+				 "entryattr.entry = entries.id "
+				 "ORDER BY entryattr.name;",
 				 (entry.category,
 				  entry.title))
 		dataSet = c.fetchAll()
@@ -821,9 +829,8 @@ class PWManDatabase(CryptSQL):
 				    silent=True)
 		return db
 
-	def dumpEntry(self, entry, showTotpKey=False):
+	def dumpEntry(self, entry, totp="hide"):
 		"""Returns a human readable dump string of an entry.
-		If showTotpKey=True: Also dump the TOTP key and parameters.
 		"""
 		res = []
 		res.append("===  %s  ===" % entry.category)
@@ -837,12 +844,21 @@ class PWManDatabase(CryptSQL):
 			res.append("\tBulk data:\t%s" % entryBulk.data)
 		entryTotp = self.getEntryTotp(entry)
 		if entryTotp:
-			if showTotpKey:
+			if totp == "show":
 				res.append("\tTOTP key:\t%s" % entryTotp.key)
 				res.append("\tTOTP digits:\t%d" % entryTotp.digits)
 				res.append("\tTOTP hash:\t%s" % entryTotp.hmacHash)
-			else:
+			elif totp == "gen":
+				try:
+					token = entryTotp.generate()
+				except libpwman.otp.OtpError as e:
+					raise PWManError("Failed to generate TOTP token: "
+							 "%s" % str(e))
+				res.append(token)
+			elif totp == "hide":
 				res.append("\tTOTP:\t\tavailable")
+			else:
+				assert False
 		entryAttrs = self.getEntryAttrs(entry)
 		if entryAttrs:
 			res.append("\tAttributes:")
@@ -855,21 +871,19 @@ class PWManDatabase(CryptSQL):
 					entryAttr.data))
 		return "\n".join(res) + "\n"
 
-	def dumpEntries(self, showTotpKey=False):
+	def dumpEntries(self, totp="hide"):
 		"""Returns a human readable dump string of all entries.
-		If showTotpKey=True: Also dump the TOTP key and parameters.
 		"""
 		ret = []
 		for category in self.getCategoryNames():
 			for title in self.getEntryTitles(category):
 				entry = self.getEntry(category, title)
-				dump = self.dumpEntry(entry, showTotpKey)
+				dump = self.dumpEntry(entry, totp)
 				ret.append(dump)
 		return "\n".join(ret)
 
-	def dumpEntriesCsv(self, showTotpKey=False):
+	def dumpEntriesCsv(self, totp="hide"):
 		"""Returns a CSV format dump string of all entries.
-		If showTotpKey=True: Also dump the TOTP key and parameters.
 		"""
 		csvHeads = [
 			"Category",
@@ -897,12 +911,21 @@ class PWManDatabase(CryptSQL):
 					row["Bulk data"] = entryBulk.data
 				entryTotp = self.getEntryTotp(entry)
 				if entryTotp:
-					if showTotpKey:
+					if totp == "show":
 						row["TOTP key"] = entryTotp.key
 						row["TOTP digits"] = entryTotp.digits
 						row["TOTP hash"] = entryTotp.hmacHash
-					else:
+					elif totp == "gen":
+						try:
+							token = entryTotp.generate()
+						except libpwman.otp.OtpError as e:
+							raise PWManError("Failed to generate TOTP token: "
+									 "%s" % str(e))
+						row["TOTP"] = token
+					elif totp == "hide":
 						row["TOTP key"] = "available"
+					else:
+						assert False
 				entryAttrs = self.getEntryAttrs(entry)
 				if entryAttrs:
 					for entryAttr in entryAttrs:
